@@ -73,7 +73,7 @@
 #define HILDON_OSSO_URI_ACTION "uri-action"
 #define URI_ACTION_COPY "copy:"
 #define MODEST_NOTIFICATION_CATEGORY "email-message"
-#define MODEST_NEW_MAIL_LIGHTING_PATTERN "PatternChatAndEmail"
+#define MODEST_NEW_MAIL_LIGHTING_PATTERN "PatternCommunicationEmail"
 
 #define COMMON_FOLDER_DIALOG_ENTRY "entry"
 #define COMMON_FOLDER_DIALOG_ACCOUNT_PICKER "account-picker"
@@ -1197,7 +1197,7 @@ update_timeout_handler (gpointer userdata)
 	}
 
 	if (auto_update && right_connection) {
-		modest_ui_actions_do_send_receive_all (NULL, FALSE, FALSE, FALSE);
+		modest_ui_actions_do_send_receive_all (NULL, FALSE, TRUE, FALSE);
 	}
 
 	return TRUE;
@@ -1276,9 +1276,12 @@ modest_platform_create_folder_view_full (TnyFolderStoreQuery *query, gboolean do
 {
 	GtkWidget *widget = modest_folder_view_new_full (query, do_refresh);
 
-	/* Show one account by default */
-	modest_folder_view_set_style (MODEST_FOLDER_VIEW (widget),
-				      MODEST_FOLDER_VIEW_STYLE_SHOW_ONE);
+	/* Show one account in list mode, all accounts in tree view mode */
+        modest_folder_view_set_style (MODEST_FOLDER_VIEW (widget),
+                                      modest_conf_get_bool (modest_runtime_get_conf (),
+                                                            MODEST_CONF_TREE_VIEW, NULL) ?
+                                      MODEST_FOLDER_VIEW_STYLE_SHOW_ALL :
+                                      MODEST_FOLDER_VIEW_STYLE_SHOW_ONE);
 
 	return widget;
 }
@@ -2366,3 +2369,29 @@ modest_platform_get_list_to_move (ModestWindow *window)
 
 	return list;
 }
+
+void
+modest_platform_emit_folder_updated_signal (const gchar *account_id, const gchar *folder_id)
+{
+       return;
+}
+
+
+void
+modest_platform_emit_account_created_signal (const gchar *account_id)
+{
+       return;
+}
+
+void
+modest_platform_emit_account_removed_signal (const gchar *account_id)
+{
+       return;
+}
+
+void
+modest_platform_emit_msg_read_changed_signal (const gchar *msg_uid,
+					      gboolean is_read)
+{
+       return;
+}
