@@ -50,6 +50,7 @@
 #include "modest-tny-msg.h"
 #include "modest-tny-account.h"
 #include <tny-merge-folder.h>
+#include <modest-account-mgr-helpers.h>
 
 /* 'private'/'protected' functions */
 static void modest_hildon2_window_mgr_class_init (ModestHildon2WindowMgrClass *klass);
@@ -421,6 +422,25 @@ modest_hildon2_window_mgr_get_window_list (ModestWindowMgr *self)
 	return g_list_copy (priv->window_list);
 }
 
+static gint window_precedence (GtkWindow *window)
+{
+	if (MODEST_IS_ACCOUNTS_WINDOW (window)) {
+		return 10;
+	} else if (MODEST_IS_MAILBOXES_WINDOW (window)) {
+		return 20;
+	} else if (MODEST_IS_FOLDER_WINDOW (window)) {
+		return 30;
+	} else if (MODEST_IS_HEADER_WINDOW (window)) {
+		return 40;
+	} else if (MODEST_IS_MSG_VIEW_WINDOW (window)) {
+		return 50;
+	} else if (MODEST_IS_MSG_EDIT_WINDOW (window)) {
+		return 60;
+	} else {
+		return 100;
+	}
+}
+
 static gboolean
 modest_hildon2_window_mgr_register_window (ModestWindowMgr *self, 
 					   ModestWindow *window,
@@ -432,7 +452,7 @@ modest_hildon2_window_mgr_register_window (ModestWindowMgr *self,
 	HildonWindowStack *stack;
 	gboolean nested_msg = FALSE;
 	ModestWindow *current_top;
-	const gchar *acc_name;
+	const gchar *acc_name, *toplevel_acc_name;
 
 	g_return_val_if_fail (MODEST_IS_HILDON2_WINDOW_MGR (self), FALSE);
 	g_return_val_if_fail (GTK_IS_WINDOW (window), FALSE);
@@ -491,38 +511,57 @@ modest_hildon2_window_mgr_register_window (ModestWindowMgr *self,
 			g_object_unref (header);
 	}
 
-	/* Do not go backwards */
-	if ((MODEST_IS_MSG_VIEW_WINDOW (current_top) ||
-	     MODEST_IS_MSG_EDIT_WINDOW (current_top) ||
-	     MODEST_IS_HEADER_WINDOW (current_top)) &&
-	    (MODEST_IS_FOLDER_WINDOW (window) ||
-	     MODEST_IS_ACCOUNTS_WINDOW (window) ||
-	     MODEST_IS_MAILBOXES_WINDOW (window))) {
-		gtk_window_present (GTK_WINDOW (current_top));
-		return FALSE;
-	}
+	/* Rules
+	 *  * toplevel = msg edit -> if not modified, close, if modified, stay
+	 *  * same account -> no action
+	 *  * window = accounts -> no action
+	 *  * window = folders, mailboxes, headers: close all up to accounts window
+	 */
 
-	if (MODEST_IS_FOLDER_WINDOW (current_top) && MODEST_IS_FOLDER_WINDOW (window)) {
+	if (MODEST_IS_MSG_EDIT_WINDOW (current_top) && 
+	    !modest_msg_edit_window_is_modified (MODEST_MSG_EDIT_WINDOW (current_top))) {
 		gboolean retval;
 
 		g_signal_emit_by_name (G_OBJECT (current_top), "delete-event", NULL, &retval);
-
-		if (retval) {
-			gtk_window_present (GTK_WINDOW (current_top));
-			return FALSE;
-		}
 		current_top = (ModestWindow *) hildon_window_stack_peek (stack);
 	}
 
-	if (MODEST_IS_MAILBOXES_WINDOW (current_top) && MODEST_IS_MAILBOXES_WINDOW (window)) {
+	if (MODEST_IS_MSG_EDIT_WINDOW (current_top) ||
+	    (current_top && MODEST_IS_ACCOUNTS_WINDOW (window))) {
 		gtk_window_present (GTK_WINDOW (current_top));
 		return FALSE;
 	}
 
-	/* Mailboxes window can not replace folder windows */
-	if (MODEST_IS_FOLDER_WINDOW (current_top) && MODEST_IS_MAILBOXES_WINDOW (window)) {
-		gtk_window_present (GTK_WINDOW (current_top));
-		return FALSE;
+	acc_name = modest_window_get_active_account (window);
+
+	if (MODEST_IS_MSG_VIEW_WINDOW (current_top) ||
+	    MODEST_IS_HEADER_WINDOW (current_top) ||
+	    MODEST_IS_FOLDER_WINDOW (current_top) ||
+	    MODEST_IS_MAILBOXES_WINDOW (current_top)) {
+		gboolean tree_view = modest_conf_get_bool (modest_runtime_get_conf (),MODEST_CONF_TREE_VIEW, NULL);
+		toplevel_acc_name = modest_window_get_active_account (current_top);
+
+		if (tree_view || (acc_name != NULL && g_strcmp0 (toplevel_acc_name, acc_name) == 0)) {
+			/* Same account or tree view, no action */
+
+			if (window_precedence (GTK_WINDOW (current_top)) >= window_precedence (GTK_WINDOW (window))) {
+				if (!(MODEST_IS_MSG_VIEW_WINDOW (current_top) && MODEST_IS_MSG_VIEW_WINDOW (window))) {
+					gtk_window_present (GTK_WINDOW (current_top));
+					return FALSE;
+				}
+			}
+		} else {
+			while (current_top && !MODEST_IS_ACCOUNTS_WINDOW (current_top)) {
+				gboolean retval;
+				g_signal_emit_by_name (G_OBJECT (current_top), "delete-event", NULL, &retval);
+
+				if (retval) {
+					gtk_window_present (GTK_WINDOW (current_top));
+					return FALSE;
+				}
+				current_top = (ModestWindow *) hildon_window_stack_peek (stack);
+			}
+		}
 	}
 
 	/* Trying to open a folders window and a mailboxes window at
@@ -584,11 +623,14 @@ modest_hildon2_window_mgr_register_window (ModestWindowMgr *self,
 
 		if (acc_name) {
 			modest_platform_remove_new_mail_notifications (FALSE, acc_name);
+			modest_account_mgr_set_has_new_mails (modest_runtime_get_account_mgr (),
+							      acc_name,
+							      FALSE);
 		}
 	}
 
 	/* Show toolbar always */
-	if (MODEST_MSG_EDIT_WINDOW (window)) {
+	if (MODEST_IS_MSG_EDIT_WINDOW (window)) {
 		gboolean show_toolbar;
 		show_toolbar = modest_conf_get_bool (modest_runtime_get_conf (), MODEST_CONF_EDIT_WINDOW_SHOW_TOOLBAR, NULL);
 		modest_window_show_toolbar (window, show_toolbar);
@@ -874,6 +916,20 @@ modest_hildon2_window_mgr_set_modal (ModestWindowMgr *self,
 	gtk_window_set_modal (window, TRUE);
 	gtk_window_set_transient_for (window, parent);
 	gtk_window_set_destroy_with_parent (window, TRUE);
+
+	if (!gtk_window_has_toplevel_focus (window)) {
+		GList *toplevels, *node;
+
+		toplevels = gtk_window_list_toplevels ();
+		for (node = toplevels; node != NULL; node = g_list_next (node)) {
+			if (gtk_window_has_toplevel_focus (GTK_WINDOW (node->data))) {
+				if (GTK_IS_DIALOG (node->data)) {					
+					gtk_window_set_transient_for (window, GTK_WINDOW (node->data));
+				}
+				break;
+			}
+		}
+	}
 }
 
 static void
@@ -961,8 +1017,15 @@ modest_hildon2_window_mgr_show_initial_window (ModestWindowMgr *self)
 							   self);
 	}
 
-	/* Return accounts window */
-	initial_window = MODEST_WINDOW (modest_accounts_window_new ());
+	/* Return initial window */
+	if (!modest_conf_get_bool (modest_runtime_get_conf (), 
+				   MODEST_CONF_TREE_VIEW, NULL))
+		initial_window = MODEST_WINDOW (modest_accounts_window_new ());
+	else {
+		initial_window = MODEST_WINDOW (modest_folder_window_new (NULL)); 
+		modest_window_set_active_account(MODEST_WINDOW(initial_window), 
+				 modest_account_mgr_get_default_account(modest_runtime_get_account_mgr()));
+	}
 	modest_window_mgr_register_window (self, initial_window, NULL);
 
 	return initial_window;
@@ -1018,7 +1081,11 @@ modest_hildon2_window_mgr_create_caches (ModestWindowMgr *self)
 {
 	g_return_if_fail (MODEST_IS_HILDON2_WINDOW_MGR (self));
 
-	modest_accounts_window_pre_create ();
+	if (modest_conf_get_bool (modest_runtime_get_conf (), 
+				  MODEST_CONF_TREE_VIEW, NULL))
+		modest_folder_window_pre_create();
+	else
+		modest_accounts_window_pre_create ();
 
 	MODEST_WINDOW_MGR_CLASS(parent_class)->create_caches (self);
 }
