@@ -36,6 +36,7 @@
 #include <strings.h>
 #include <modest-account-protocol.h>
 #include <modest-utils.h>
+#include <modest-platform.h>
 
 static const gchar * null_means_empty (const gchar * str);
 
@@ -49,7 +50,16 @@ gboolean
 modest_account_mgr_set_enabled (ModestAccountMgr *self, const gchar* name,
 					gboolean enabled)
 {
-	return modest_account_mgr_set_bool (self, name, MODEST_ACCOUNT_ENABLED, enabled,FALSE);
+	gboolean result;
+	result = modest_account_mgr_set_bool (self, name, MODEST_ACCOUNT_ENABLED, enabled,FALSE);
+	if (result) {
+		if (enabled) {
+			modest_platform_emit_account_created_signal (name);
+		} else {
+			modest_platform_emit_account_removed_signal (name);
+		}
+	}
+	return result;
 }
 
 
@@ -458,6 +468,8 @@ modest_account_mgr_load_server_settings (ModestAccountMgr *self, const gchar* na
 	ModestProtocol *protocol;
 	ModestProtocolRegistry *registry;
 	gchar *hostname, *username, *pwd, *uri, *proto, *auth, *sec;
+        gboolean offline_sync;
+	gboolean update_all_folders;
 
 	if (!modest_account_mgr_account_exists (self, name, TRUE)) {
 		g_warning ("%s account %s does not exist", __FUNCTION__, name);
@@ -535,6 +547,17 @@ modest_account_mgr_load_server_settings (ModestAccountMgr *self, const gchar* na
 	if (hostname)
 		modest_server_account_settings_set_hostname (settings, hostname);
 
+        offline_sync = modest_account_mgr_get_bool (self, name, 
+                                       		    MODEST_ACCOUNT_OFFLINE_SYNC, TRUE);
+        if (offline_sync)
+            modest_server_account_settings_set_offline_sync (settings, TRUE);
+
+	update_all_folders = modest_account_mgr_get_bool (self, name,
+							  MODEST_ACCOUNT_UPDATE_ALL_FOLDERS, TRUE);
+
+	if (update_all_folders)
+		modest_server_account_settings_set_update_all_folders (settings, TRUE);
+
 	if (!uri) {
 		if (!username || !hostname) {
 			g_free (username);
@@ -584,6 +607,8 @@ modest_account_mgr_save_server_settings (ModestAccountMgr *self,
 		gint port;
 		const gchar *auth_protocol_name;
 		const gchar *security_name;
+                gboolean offline_sync;
+		gboolean update_all_folders;
 
 		hostname = null_means_empty (modest_server_account_settings_get_hostname (settings));
 		username = null_means_empty (modest_server_account_settings_get_username (settings));
@@ -595,6 +620,8 @@ modest_account_mgr_save_server_settings (ModestAccountMgr *self,
 		protocol = modest_protocol_registry_get_protocol_by_type (protocol_registry,
 									  modest_server_account_settings_get_security_protocol (settings));
 		security_name = modest_protocol_get_name (protocol);
+                offline_sync = modest_server_account_settings_get_offline_sync (settings);
+		update_all_folders = modest_server_account_settings_get_update_all_folders (settings);
 
 		has_errors = !modest_account_mgr_set_string (self, account_name, MODEST_ACCOUNT_HOSTNAME, 
 							    hostname, TRUE);
@@ -618,6 +645,14 @@ modest_account_mgr_save_server_settings (ModestAccountMgr *self,
 			(has_errors = !modest_account_mgr_set_string (self, account_name, MODEST_ACCOUNT_SECURITY,
 									   security_name,
 									   TRUE));
+		if (!has_errors)
+			(has_errors = !modest_account_mgr_set_bool (self, account_name, MODEST_ACCOUNT_OFFLINE_SYNC,
+									    offline_sync,
+									    TRUE));
+		if (!has_errors)
+			(has_errors = !modest_account_mgr_set_bool (self, account_name, MODEST_ACCOUNT_UPDATE_ALL_FOLDERS,
+									    update_all_folders,
+									    TRUE));
 	} else {
 		const gchar *uri = modest_server_account_settings_get_uri (settings);
 		has_errors = !modest_account_mgr_set_string (self, account_name, MODEST_ACCOUNT_URI,
@@ -790,7 +825,7 @@ modest_account_mgr_save_account_settings (ModestAccountMgr *mgr,
 		modest_account_mgr_save_server_settings (mgr, transport_settings);
 		g_object_unref (transport_settings);
 	}
-	modest_account_mgr_set_enabled (mgr, account_name, TRUE);
+	modest_account_mgr_set_bool (mgr, account_name, MODEST_ACCOUNT_ENABLED, TRUE,FALSE);
 }
 
 
@@ -1034,6 +1069,36 @@ modest_account_mgr_set_last_updated (ModestAccountMgr *self,
 				    time, 
 				    TRUE);
 
+	/* if 'account_name' is not defined, use "<null>" string */
+	if (!account_name) {
+		account_name = "<null>";
+	}
+
+	/* send a folder-updated signal, using a dummi folder id */
+	modest_platform_emit_folder_updated_signal (account_name, "inbox");
+}
+
+gboolean
+modest_account_mgr_get_has_new_mails (ModestAccountMgr *self, 
+				      const gchar* account_name)
+{
+	return modest_account_mgr_get_bool (modest_runtime_get_account_mgr (), 
+					    account_name, 
+					    MODEST_ACCOUNT_HAS_NEW_MAILS, 
+					    FALSE);
+}
+
+void 
+modest_account_mgr_set_has_new_mails (ModestAccountMgr *self, 
+				      const gchar* account_name,
+				      gboolean has_new_mails)
+{
+	modest_account_mgr_set_bool (self, 
+				     account_name, 
+				     MODEST_ACCOUNT_HAS_NEW_MAILS, 
+				     has_new_mails, 
+				     FALSE);
+
 	/* TODO: notify about changes */
 }
 
