@@ -44,18 +44,20 @@
 #include <modest-account-mgr-helpers.h>
 #include <modest-runtime.h>
 #include <ctype.h>
+#include <locale.h>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H */
 
 /* defines */
-#define FORWARD_STRING _("mcen_ia_editor_original_message")
-#define FROM_STRING _("mail_va_from")
-#define SENT_STRING _("mcen_fi_message_properties_sent")
-#define TO_STRING _("mail_va_to")
-#define	SUBJECT_STRING _("mail_va_subject")
+#define FORWARD_STRING "----- Original message -----"
+#define FROM_STRING "From"
+#define SENT_STRING "Sent"
+#define TO_STRING "To"
+#define SUBJECT_STRING "Subject"
 #define EMPTY_STRING ""
+#define SEPARATOR_STRING _HL("ecdg_ti_caption_separator")
 
 /*
  * do the hyperlinkification only for texts < 50 Kb,
@@ -239,14 +241,22 @@ forward_cite (const gchar *from,
 	      const gchar *to,
 	      const gchar *subject)
 {
+	const gchar *separator;
+
 	g_return_val_if_fail (sent, NULL);
-	
-	return g_strdup_printf ("%s\n%s %s\n%s %s\n%s %s\n%s %s\n", 
-				FORWARD_STRING, 
-				FROM_STRING, (from)?from:"",
-				SENT_STRING, sent,
-				TO_STRING, (to)?to:"",
-				SUBJECT_STRING, (subject)?subject:"");
+
+	/* the separator string/char (':') */
+	separator = SEPARATOR_STRING;
+	return g_strdup_printf ("%s\n" /* original message marker */
+		"%s%s %s\n" /* original 'From' string */
+		"%s%s %s\n" /* original 'Sent' string (date/time) */
+		"%s%s %s\n" /* original 'To' string */
+		"%s%s %s", /* original 'Subject' string */
+		FORWARD_STRING,
+		FROM_STRING, separator, from ? from:EMPTY_STRING,
+		SENT_STRING, separator, sent,
+		TO_STRING, separator, to ? to : EMPTY_STRING,
+		SUBJECT_STRING, separator, subject ? subject : EMPTY_STRING);
 }
 
 gchar * 
@@ -261,11 +271,15 @@ modest_text_utils_inline (const gchar *text,
 	gchar sent_str[101];
 	gchar *cited;
 	gchar *retval;
+	char *locale;
 	
 	g_return_val_if_fail (text, NULL);
 	g_return_val_if_fail (content_type, NULL);
 	
+	locale = setlocale(LC_TIME, NULL);
+	setlocale(LC_TIME, "C");
 	modest_text_utils_strftime (sent_str, 100, "%c", sent_date);
+	setlocale(LC_TIME, locale);
 
 	cited = forward_cite (from, sent_str, to, subject);
 	
@@ -542,6 +556,11 @@ modest_text_utils_convert_buffer_to_html_start (GString *html, const gchar *data
 			g_string_append_c (html, kar);
 		}
 	}
+
+	/* check if the last char in the 'data' is a space */
+	if (space_seen) {
+		g_string_append (html, "&#32;");
+	}
 }
 
 
@@ -615,7 +634,6 @@ modest_text_utils_convert_to_html_body (const gchar *data, gssize n, gboolean hy
 void
 modest_text_utils_get_addresses_indexes (const gchar *addresses, GSList **start_indexes, GSList **end_indexes)
 {
-	GString *str;
 	gchar *start, *cur;
 
 	if (!addresses)
@@ -624,7 +642,6 @@ modest_text_utils_get_addresses_indexes (const gchar *addresses, GSList **start_
 	if (strlen (addresses) == 0)
 		return;
 
-	str = g_string_new ("");
 	start = (gchar*) addresses;
 	cur = (gchar*) addresses;
 
@@ -883,9 +900,11 @@ unquote_line (GString * l, const gchar *quote_symbol)
 	quote_len = strlen (quote_symbol);
 	while (p[0]) {
 		if (g_str_has_prefix (p, quote_symbol)) {
-			if (p[quote_len] == ' ') {
-				p += quote_len;
+			p+=quote_len;
+			while (p[0] && p[0] == ' ') {
+				p++;
 			}
+			continue;
 		} else {
 			break;
 		}
@@ -982,7 +1001,12 @@ get_breakpoint (const gchar * s, const gint indent, const gint limit)
 static gchar *
 cite (const time_t sent_date, const gchar *from)
 {
-	return g_strdup (_("mcen_ia_editor_original_message"));
+	char sent_str[101];
+	char *locale = setlocale(LC_TIME, NULL);
+	setlocale(LC_TIME, "C");
+	modest_text_utils_strftime (sent_str, 100, "%c", sent_date);
+	setlocale(LC_TIME, locale);
+	return g_strdup_printf("On %s %s wrote:", sent_str, from);
 }
 
 static gchar *
@@ -1007,7 +1031,7 @@ modest_text_utils_quote_body (GString *output, const gchar *text,
 
 	const gchar *iter;
 	gsize len;
-	gint indent, breakpoint, rem_indent = 0;
+	gint indent = 0, breakpoint, rem_indent = 0;
 	GString *l, *remaining;
 	gchar *forced_wrap_append;
 
@@ -1106,17 +1130,16 @@ modest_text_utils_quote_plain_text (const gchar *text,
 
 	q = g_string_new ("");
 
-	if (signature != NULL) {
-		g_string_append_printf (q, "\n%s\n", MODEST_TEXT_UTILS_SIGNATURE_MARKER);
-		q = g_string_append (q, signature);
-	}
-
-	q = g_string_append (q, "\n");
 	q = g_string_append (q, cite);
 	q = g_string_append_c (q, '\n');
 
 	q = modest_text_utils_quote_body (q, text, ">", limit);
 
+	if (signature != NULL) {
+		g_string_append_printf (q, "\n%s\n", MODEST_TEXT_UTILS_SIGNATURE_MARKER);
+		q = g_string_append (q, signature);
+	}
+
 	attachments_string = quoted_attachments (attachments);
 	q = g_string_append (q, attachments_string);
 	g_free (attachments_string);
@@ -2313,19 +2336,100 @@ modest_text_utils_create_colored_signature (const gchar *signature)
 	gchar *gray_color_markup = NULL, *retval;
 	GdkColor color;
 	GtkWidget *widget;
+	GString *html_signature;
+	GString *html_signature_marker;
 
 	/* Get color from widgets */
 	widget = (GtkWidget *) modest_window_mgr_get_current_top (modest_runtime_get_window_mgr ());
 	if (widget && gtk_style_lookup_color (gtk_widget_get_style (widget), "SecondaryTextColor", &color))
 		gray_color_markup = modest_text_utils_get_color_string (&color);
 
+	/* convert the signature to HTML format */
+	html_signature = g_string_new ("");
+	modest_text_utils_convert_buffer_to_html_start (html_signature, signature, -1);
+	modest_text_utils_convert_buffer_to_html_finish (html_signature);
+
+	/* get the signature marker */
+	html_signature_marker = g_string_new ("");
+	modest_text_utils_convert_buffer_to_html_start (html_signature_marker,
+		MODEST_TEXT_UTILS_SIGNATURE_MARKER, -1);
+	modest_text_utils_convert_buffer_to_html_finish (html_signature_marker);
+
 	retval = g_strdup_printf ("<br/>\n<font color=\"%s\">%s<br/>\n%s<br/>\n</font>",
 				  (gray_color_markup) ? gray_color_markup : "#babababababa",
-				  MODEST_TEXT_UTILS_SIGNATURE_MARKER,
-				  signature);
+				  html_signature_marker->str,
+				  html_signature->str);
 
+	g_string_free (html_signature_marker, TRUE);
+	g_string_free (html_signature, TRUE);
 	if (gray_color_markup)
 		g_free (gray_color_markup);
 
 	return retval;
 }
+
+gboolean
+modest_text_utils_live_search_find (const gchar *haystack, const gchar *needles)
+{
+	gchar *haystack_fold;
+	gchar *needles_fold;
+	gchar **needle, **current;
+	gboolean match;
+
+	match = FALSE;
+
+	haystack_fold = g_utf8_casefold (haystack, -1);
+	needles_fold = g_utf8_casefold (needles, -1);
+
+	needle = g_strsplit (needles_fold, " ", -1);
+
+	current = needle;
+	while (current && *current != NULL) {
+
+		if (g_strstr_len (haystack_fold, -1, *current) != NULL) {
+			match = TRUE;
+		} else {
+			match = FALSE;
+			break;
+		}
+		current++;
+	}
+
+	g_strfreev (needle);
+	g_free (needles_fold);
+	g_free (haystack_fold);
+	
+	return match;
+}
+
+gchar* modest_text_utils_create_filename (const gchar *filename)
+{
+	gchar *ptr;
+	gchar *result;
+	static const gchar invalid_characters[] = { " ()[]{}<>|\\/:;=+\"`'&$%!*?" };
+
+	result = g_strndup (filename, 100);
+	ptr = result;
+	while (ptr) {
+		gunichar ch;
+		const gchar *invalid;
+
+		/* get the current character */
+		ch = g_utf8_get_char (ptr);
+		if (!ch) {
+			break;
+		}
+
+		/* check if the current character is valid */
+		invalid = g_utf8_strchr (invalid_characters, -1, ch);
+		if (invalid) {
+			/* all exceptions are 1-byte sequences, substitute in-place */
+			*ptr = '_';
+		}
+
+		/* move on to the next character */
+		ptr = g_utf8_next_char (ptr);
+	}
+
+	return result;
+}
