@@ -106,7 +106,7 @@
 #define _FM_REPLACE_FILE _FM("docm_nc_replace_file")
 #define _FM_REPLACE_MULTIPLE _FM("docm_nc_replace_multiple")
 #define _FM_READ_ONLY_LOCATION _FM("sfil_ib_readonly_location")
-#define _FM_SAVE_OBJECT_FILES _FM("sfil_ti_save_object_files")
+#define _FM_SAVE_OBJECT_FILES _FM("mcen_me_viewer_save_attachments")
 #define _FM_CHANGE_FOLDER_NEW_FOLDER ("ckdg_bd_change_folder_new_folder")
 #else
 #define _HL_SAVE _("Save")
@@ -697,4 +697,16 @@ gboolean modest_text_utils_no_recipient (GtkTextBuffer *buffer);
 
 gchar * modest_text_utils_create_colored_signature (const gchar *signature);
 
+gboolean modest_text_utils_live_search_find (const gchar *haystack, const gchar *needles);
+
+/**
+ * Create a valid filename based on the filename
+ * @param[in] filename The oridinal filename, it might not be a valid filename
+ * @return The allocated valid filename. Should be released with 'g_free ()'
+ * @note This function allocates a new string that confirms to the filename rules.
+ *       All invalid characters are substituted to '_'.
+ *       For example: "name!/.txt" -> "name__.txt"
+ */
+gchar* modest_text_utils_create_filename (const gchar *filename);
+
 #endif /* __MODEST_TEXT_UTILS_H__ */
