@@ -207,6 +207,7 @@ create_tny_account (TnySessionCamel *session,
 /* Camel options: */
 
 #define MODEST_ACCOUNT_OPTION_SSL "use_ssl"
+#define MODEST_ACCOUNT_OPTION_OFFLINE_SYNC "offline_sync"
 
 
 		
@@ -246,6 +247,7 @@ update_tny_account (TnyAccount *tny_account,
 		/* Set camel-specific options: */		
 		/* Enable secure connection settings: */
 		TnyPair *option_security = NULL;
+		TnyPair *option_offline_sync = NULL;
 		const gchar* auth_mech_name = NULL;
 		ModestProtocolType protocol_type;
 		ModestProtocol *protocol;
@@ -258,6 +260,8 @@ update_tny_account (TnyAccount *tny_account,
 		const gchar *username;
 		const gchar *hostname;
 		guint port;
+		gboolean offline_sync;
+		gboolean update_all_folders;
 
 		/* First of all delete old options */
 		tny_camel_account_clear_options (TNY_CAMEL_ACCOUNT (tny_account));
@@ -269,6 +273,16 @@ update_tny_account (TnyAccount *tny_account,
 		protocol = modest_protocol_registry_get_protocol_by_type (protocol_registry, protocol_type);
 		security = modest_protocol_registry_get_protocol_by_type (protocol_registry, security_type);
 		auth_protocol = modest_protocol_registry_get_protocol_by_type (protocol_registry, auth_protocol_type);
+		offline_sync = modest_server_account_settings_get_offline_sync (server_settings);
+		update_all_folders = modest_server_account_settings_get_update_all_folders (server_settings);
+
+		if (offline_sync) {
+			option_offline_sync = tny_pair_new (MODEST_ACCOUNT_OPTION_OFFLINE_SYNC, "");
+			tny_camel_account_add_option (TNY_CAMEL_ACCOUNT (tny_account), option_offline_sync);
+			g_object_unref (option_offline_sync);
+		}
+
+		g_object_set_data (G_OBJECT(tny_account), MODEST_ACCOUNT_UPDATE_ALL_FOLDERS, (gpointer)update_all_folders);
 
 		security_option_string = modest_protocol_get (security, MODEST_PROTOCOL_SECURITY_ACCOUNT_OPTION);
 		if (security_option_string) {
@@ -575,7 +589,7 @@ modest_tny_account_new_from_account (ModestAccountMgr *account_mgr,
 	tny_account_set_connection_policy (tny_account, policy);
 	g_object_unref (policy);
 
-        modest_tny_account_set_parent_modest_account_name_for_server_account (tny_account,
+	modest_tny_account_set_parent_modest_account_name_for_server_account (tny_account,
 									      account_name);
 	g_object_unref (server_settings);
 	g_object_unref (settings);
