@@ -374,7 +374,7 @@ restore_settings_header_view (ModestConf *conf, ModestHeaderView *header_view,
 		GList *cols = NULL;
 		GList *colwidths = NULL;
 		GList *colsortables = NULL;
-		GtkTreeModel *sortable;
+		GtkTreeModel *sortable, *filter;
 
 		cursor = data = modest_conf_get_string (conf, key, NULL);
 		while (cursor && sscanf (cursor, "%d:%d:%d ", &col, &width, &sort) == 3) {
@@ -401,7 +401,8 @@ restore_settings_header_view (ModestConf *conf, ModestHeaderView *header_view,
 		if (cols) {
 			GList *viewcolumns, *colcursor, *widthcursor;
 			modest_header_view_set_columns (header_view, cols, type);
-			sortable = gtk_tree_view_get_model (GTK_TREE_VIEW (header_view));
+			filter = gtk_tree_view_get_model (GTK_TREE_VIEW (header_view));
+			sortable = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filter));
 
 			widthcursor = colwidths;
 			colcursor = viewcolumns = gtk_tree_view_get_columns (GTK_TREE_VIEW(header_view));
@@ -430,7 +431,8 @@ restore_settings_header_view (ModestConf *conf, ModestHeaderView *header_view,
 	}
 
 	if (sort_colid >= 0) {
-		GtkTreeModel *sortable = gtk_tree_view_get_model (GTK_TREE_VIEW (header_view));
+		GtkTreeModel *filter_model = gtk_tree_view_get_model (GTK_TREE_VIEW (header_view));
+		GtkTreeModel *sortable = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filter_model));
 		if (sort_colid == TNY_GTK_HEADER_LIST_MODEL_FLAGS_COLUMN)
 			modest_header_view_sort_by_column_id (header_view, 0, sort_type);
 		gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (sortable),
