@@ -44,6 +44,7 @@
 #include <modest-ui-constants.h>
 #ifdef MODEST_TOOLKIT_HILDON2
 #include <hildon/hildon-defines.h>
+#include <hildon/hildon.h>
 #endif
 #ifdef MODEST_USE_LIBTIME
 #include <clockd/libtime.h>
@@ -68,6 +69,7 @@ static void on_account_updated (ModestAccountMgr* mgr, gchar* account_name,
                     gpointer user_data);
 static void update_account_view (ModestAccountMgr *account_mgr, ModestAccountView *view);
 static void on_notify_style (GObject *obj, GParamSpec *spec, gpointer userdata);
+static void update_style (ModestAccountView *self);
 static void update_display_mode (ModestAccountView *self);
 
 typedef enum {
@@ -77,6 +79,7 @@ typedef enum {
 	MODEST_ACCOUNT_VIEW_IS_DEFAULT_COLUMN,
 	MODEST_ACCOUNT_VIEW_PROTO_COLUMN,
 	MODEST_ACCOUNT_VIEW_LAST_UPDATED_COLUMN,
+	MODEST_ACCOUNT_VIEW_HAS_NEW_MAILS_COLUMN,
 
 	MODEST_ACCOUNT_VIEW_COLUMN_NUM
 } AccountViewColumns;
@@ -89,8 +92,16 @@ struct _ModestAccountViewPrivate {
 	gboolean picker_mode;
 	gboolean show_last_updated;
 
+	GtkTreeModel *model;
+	GtkTreeModel *filter;
+
 	/* Signal handlers */
 	GSList *sig_handlers;
+
+	GtkTreeViewColumn *account_name_column;
+	GtkCellRenderer *account_name_renderer;
+
+	guint update_account_view_idle_id;
 };
 #define MODEST_ACCOUNT_VIEW_GET_PRIVATE(o)      (G_TYPE_INSTANCE_GET_PRIVATE((o), \
                                                  MODEST_TYPE_ACCOUNT_VIEW, \
@@ -152,6 +163,7 @@ modest_account_view_init (ModestAccountView *obj)
 	priv = MODEST_ACCOUNT_VIEW_GET_PRIVATE(obj);
 	
 	priv->sig_handlers = NULL;
+	priv->update_account_view_idle_id = 0;
 
 	priv->datetime_formatter = modest_datetime_formatter_new ();
 	priv->picker_mode = FALSE;
@@ -175,6 +187,11 @@ modest_account_view_finalize (GObject *obj)
 
 	priv = MODEST_ACCOUNT_VIEW_GET_PRIVATE(obj);
 
+	if (priv->update_account_view_idle_id > 0) {
+		g_source_remove (priv->update_account_view_idle_id);
+		priv->update_account_view_idle_id = 0;
+	}
+
 	if (priv->datetime_formatter) {
 		g_object_unref (priv->datetime_formatter);
 		priv->datetime_formatter = NULL;
@@ -225,15 +242,17 @@ get_last_updated_string(ModestAccountView *self, ModestAccountMgr* account_mgr,
 	return last_updated_string;
 }
 
-static void
-update_account_view (ModestAccountMgr *account_mgr, ModestAccountView *view)
+static gboolean
+update_account_view_idle (gpointer userdata)
 {
-	GSList *account_names, *cursor;
-	GtkListStore *model;
+	ModestAccountView *view;
  	ModestAccountViewPrivate *priv;
-	
+	ModestAccountMgr *account_mgr;
+	GSList *account_names, *cursor;
+
+	view = MODEST_ACCOUNT_VIEW (userdata);
 	priv = MODEST_ACCOUNT_VIEW_GET_PRIVATE(view);
-	model = GTK_LIST_STORE(gtk_tree_view_get_model (GTK_TREE_VIEW(view)));
+	account_mgr = modest_runtime_get_account_mgr ();
 
 	/* Get the ID of the currently-selected account,
 	 * so we can select it again after rebuilding the list.
@@ -241,7 +260,7 @@ update_account_view (ModestAccountMgr *account_mgr, ModestAccountView *view)
 	 */
 	gchar *selected_name = modest_account_view_get_selected_account (view);
 
-	gtk_list_store_clear (model);
+	gtk_list_store_clear (GTK_LIST_STORE (priv->model));
 
 	/* Note: We do not show disabled accounts.
 	 * Of course, this means that there is no UI to enable or disable
@@ -275,6 +294,7 @@ update_account_view (ModestAccountMgr *account_mgr, ModestAccountView *view)
 
 			/* don't free */
 			const gchar *last_updated_string = get_last_updated_string(view, account_mgr, settings);
+			gboolean has_new_mails = modest_account_mgr_get_has_new_mails (account_mgr, account_name);
 			
 			if (modest_account_settings_get_enabled (settings)) {
 				ModestProtocolType protocol_type;
@@ -295,7 +315,7 @@ update_account_view (ModestAccountMgr *account_mgr, ModestAccountView *view)
 				protocol = modest_protocol_registry_get_protocol_by_type (protocol_registry, protocol_type);
 				proto_name = modest_protocol_get_name (protocol);
 				gtk_list_store_insert_with_values (
-					model, &iter, 0,
+					GTK_LIST_STORE (priv->model), &iter, 0,
 					MODEST_ACCOUNT_VIEW_NAME_COLUMN, account_name,
 					MODEST_ACCOUNT_VIEW_DISPLAY_NAME_COLUMN, 
 					modest_account_settings_get_display_name (settings),
@@ -305,6 +325,7 @@ update_account_view (ModestAccountMgr *account_mgr, ModestAccountView *view)
 					modest_account_settings_get_is_default (settings),
 					MODEST_ACCOUNT_VIEW_PROTO_COLUMN, proto_name,
 					MODEST_ACCOUNT_VIEW_LAST_UPDATED_COLUMN,  last_updated_hildon2,
+					MODEST_ACCOUNT_VIEW_HAS_NEW_MAILS_COLUMN, has_new_mails,
 					-1);
 				g_free (last_updated_hildon2);
 			}
@@ -323,24 +344,41 @@ update_account_view (ModestAccountMgr *account_mgr, ModestAccountView *view)
 		modest_account_view_select_account (view, selected_name);
 		g_free (selected_name);
 	}
+
+	priv->update_account_view_idle_id = 0;
+	return FALSE;
 }
 
+static void
+update_account_view (ModestAccountMgr *account_mgr, ModestAccountView *view)
+{
+ 	ModestAccountViewPrivate *priv;
+	priv = MODEST_ACCOUNT_VIEW_GET_PRIVATE(view);
+
+	if (priv->update_account_view_idle_id == 0) {
+		priv->update_account_view_idle_id = g_idle_add (update_account_view_idle, (gpointer) view);
+	}
+}
+
+
 static void
 on_account_busy_changed(ModestAccountMgr *account_mgr, 
                         const gchar *account_name,
                         gboolean busy, 
                         ModestAccountView *self)
 {
-	GtkListStore *model = GTK_LIST_STORE(gtk_tree_view_get_model (GTK_TREE_VIEW(self)));
 	GtkTreeIter iter;
 	gboolean found = FALSE;
+	ModestAccountViewPrivate *priv;
 
-	if (!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter))
+	priv = MODEST_ACCOUNT_VIEW_GET_PRIVATE(self);
+
+	if (!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(priv->model), &iter))
 		return;
 
 	do {
 		gchar* cur_name;
-		gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
+		gtk_tree_model_get(GTK_TREE_MODEL(priv->model), &iter, 
 				   MODEST_ACCOUNT_VIEW_NAME_COLUMN, 
 				   &cur_name, -1);
 
@@ -357,7 +395,7 @@ on_account_busy_changed(ModestAccountMgr *account_mgr,
 			last_updated_hildon2 = g_strconcat (_("mcen_ti_lastupdated"), "\n", 
 							    last_updated_string,
 							    NULL);
-			gtk_list_store_set(model, &iter, 
+			gtk_list_store_set(GTK_LIST_STORE (priv->model), &iter, 
 					   MODEST_ACCOUNT_VIEW_LAST_UPDATED_COLUMN, last_updated_hildon2,
 					   -1);
 
@@ -367,7 +405,7 @@ on_account_busy_changed(ModestAccountMgr *account_mgr,
 		}
 		g_free (cur_name);
 
-	} while (!found && gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter));
+	} while (!found && gtk_tree_model_iter_next(priv->model, &iter));
 }
 
 static void
@@ -422,7 +460,6 @@ on_account_default_toggled (GtkCellRendererToggle *cell_renderer,
 			    ModestAccountView *self)
 {
 	ModestAccountViewPrivate *priv;
-	GtkTreeModel *model;
 	GtkTreeIter iter;
 	gchar *account_name = NULL;
 
@@ -434,14 +471,13 @@ on_account_default_toggled (GtkCellRendererToggle *cell_renderer,
 		return;
 
 	priv = MODEST_ACCOUNT_VIEW_GET_PRIVATE(self);
-	model = gtk_tree_view_get_model (GTK_TREE_VIEW(self));
 
-	if (!gtk_tree_model_get_iter_from_string (model, &iter, path)) {
+	if (!gtk_tree_model_get_iter_from_string (priv->filter, &iter, path)) {
 		g_warning ("Got path of a not existing iter");
 		return;
 	}
 	
-	gtk_tree_model_get (model, &iter, 
+	gtk_tree_model_get (priv->filter, &iter, 
 			    MODEST_ACCOUNT_VIEW_NAME_COLUMN, 
 			    &account_name, -1);
 
@@ -477,8 +513,9 @@ bold_if_default_account_cell_data  (GtkTreeViewColumn *column,  GtkCellRenderer
 	PangoAttrList *attr_list = NULL;
 	GtkWidget *widget;
 
-	gtk_tree_model_get (tree_model, iter, MODEST_ACCOUNT_VIEW_IS_DEFAULT_COLUMN,
-			    &is_default, -1);
+	gtk_tree_model_get (tree_model, iter,
+			    MODEST_ACCOUNT_VIEW_IS_DEFAULT_COLUMN, &is_default,
+			    -1);
 
 /* 	widget = gtk_tree_view_column_get_tree_view (column); */
 	widget = GTK_WIDGET (user_data);
@@ -547,27 +584,31 @@ init_view (ModestAccountView *self)
 {
 	ModestAccountViewPrivate *priv;
 	GtkCellRenderer *toggle_renderer, *text_renderer;
-	GtkListStore *model;
 	GtkTreeViewColumn *column;
 	
 	g_return_if_fail (MODEST_IS_ACCOUNT_VIEW (self));
 	priv = MODEST_ACCOUNT_VIEW_GET_PRIVATE(self);
 		
-	model = gtk_list_store_new (6,
-				    G_TYPE_STRING,  /* account name */
-				    G_TYPE_STRING,  /* account display name */
-				    G_TYPE_BOOLEAN, /* is-enabled */
-				    G_TYPE_BOOLEAN, /* is-default */
-				    G_TYPE_STRING,  /* account proto (pop, imap,...) */
-				    G_TYPE_STRING   /* last updated (time_t) */
-		); 
+	priv->model = GTK_TREE_MODEL (gtk_list_store_new (8,
+							  G_TYPE_STRING,  /* account name */
+							  G_TYPE_STRING,  /* account display name */
+							  G_TYPE_BOOLEAN, /* is-enabled */
+							  G_TYPE_BOOLEAN, /* is-default */
+							  G_TYPE_STRING,  /* account proto (pop, imap,...) */
+							  G_TYPE_STRING,   /* last updated (time_t) */
+							  G_TYPE_BOOLEAN, /* has_new_mails has_color */
+							  G_TYPE_POINTER /* has_new_mails color */
+					      )); 
 		
 	gtk_tree_sortable_set_sort_column_id (
-		GTK_TREE_SORTABLE (model), MODEST_ACCOUNT_VIEW_DISPLAY_NAME_COLUMN, 
+		GTK_TREE_SORTABLE (priv->model), MODEST_ACCOUNT_VIEW_DISPLAY_NAME_COLUMN, 
 		GTK_SORT_ASCENDING);
 
-	gtk_tree_view_set_model (GTK_TREE_VIEW(self), GTK_TREE_MODEL(model));
-	g_object_unref (G_OBJECT (model));
+	priv->filter = gtk_tree_model_filter_new (priv->model, NULL);
+
+	gtk_tree_view_set_model (GTK_TREE_VIEW(self), GTK_TREE_MODEL(priv->filter));
+	g_object_unref (G_OBJECT (priv->model));
+	g_object_unref (G_OBJECT (priv->filter));
 
 	toggle_renderer = gtk_cell_renderer_toggle_new ();
 	/* the is_default column */
@@ -595,17 +636,19 @@ init_view (ModestAccountView *self)
 					   self);
 	
 	/* account name */
-	text_renderer = gtk_cell_renderer_text_new ();
-	g_object_set (G_OBJECT (text_renderer), 
+	priv->account_name_renderer = gtk_cell_renderer_text_new ();
+	g_object_set (G_OBJECT (priv->account_name_renderer), 
 		      "ellipsize", PANGO_ELLIPSIZE_END, "ellipsize-set", TRUE, 
 		      "xpad", MODEST_MARGIN_DOUBLE,
 		      NULL);
 
-	column =  gtk_tree_view_column_new_with_attributes (_("mcen_ti_account"), text_renderer, "text",
-							    MODEST_ACCOUNT_VIEW_DISPLAY_NAME_COLUMN, NULL);
-	gtk_tree_view_append_column (GTK_TREE_VIEW(self), column);
-	gtk_tree_view_column_set_expand (column, TRUE);
-	gtk_tree_view_column_set_cell_data_func(column, text_renderer, bold_if_default_account_cell_data,
+	priv->account_name_column =  gtk_tree_view_column_new_with_attributes (_("mcen_ti_account"), priv->account_name_renderer,
+									       "text", MODEST_ACCOUNT_VIEW_DISPLAY_NAME_COLUMN,
+									       NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW(self), priv->account_name_column);
+	gtk_tree_view_column_set_expand (priv->account_name_column, TRUE);
+	gtk_tree_view_column_set_cell_data_func(priv->account_name_column, priv->account_name_renderer, 
+						bold_if_default_account_cell_data,
 						self, NULL);
 
 	/* last update for this account */
@@ -802,6 +845,7 @@ static void
 on_notify_style (GObject *obj, GParamSpec *spec, gpointer userdata)
 {
 	if (strcmp ("style", spec->name) == 0) {
+		update_style (MODEST_ACCOUNT_VIEW (obj));
 		gtk_widget_queue_draw (GTK_WIDGET (obj));
 	} 
 }
@@ -878,3 +922,89 @@ modest_account_view_get_show_last_updated (ModestAccountView *self)
 
 	return priv->show_last_updated;
 }
+
+GtkTreeModel *
+modest_account_view_get_model (ModestAccountView *self)
+{
+	ModestAccountViewPrivate *priv;
+
+	priv = MODEST_ACCOUNT_VIEW_GET_PRIVATE (self);
+
+	return priv->model;
+}
+
+GtkTreeModel *
+modest_account_view_get_filter (ModestAccountView *self)
+{
+	ModestAccountViewPrivate *priv;
+
+	priv = MODEST_ACCOUNT_VIEW_GET_PRIVATE (self);
+
+	return priv->filter;
+}
+
+#ifdef MODEST_TOOLKIT_HILDON2
+static gboolean
+live_search_visible_func (GtkTreeModel *model,
+			  GtkTreeIter  *iter,
+			  gchar        *text,
+			  gpointer      data)
+{
+	gchar *display_name;
+	gboolean result;
+	gtk_tree_model_get(model, iter, 
+			   MODEST_ACCOUNT_VIEW_DISPLAY_NAME_COLUMN,
+			   &display_name, -1);
+
+	result = modest_text_utils_live_search_find (display_name, text);
+
+	g_free (display_name);
+
+	return result;
+}
+
+GtkWidget *
+modest_account_view_setup_live_search (ModestAccountView *self)
+{
+	GtkWidget *live_search;
+	live_search = hildon_live_search_new ();
+	hildon_live_search_set_filter (HILDON_LIVE_SEARCH (live_search),
+				       GTK_TREE_MODEL_FILTER (modest_account_view_get_filter (MODEST_ACCOUNT_VIEW (self))));
+	hildon_live_search_set_visible_func (HILDON_LIVE_SEARCH (live_search), live_search_visible_func, self, NULL);
+	hildon_live_search_set_text_column (HILDON_LIVE_SEARCH (live_search),
+					    MODEST_ACCOUNT_VIEW_DISPLAY_NAME_COLUMN);
+
+	return live_search;
+}
+#endif
+
+static void
+update_style (ModestAccountView *self)
+{
+	ModestAccountViewPrivate *priv;
+	GdkColor style_active_color;
+
+	g_return_if_fail (MODEST_IS_ACCOUNT_VIEW (self));
+	priv = MODEST_ACCOUNT_VIEW_GET_PRIVATE (self);
+
+	if (gtk_style_lookup_color (GTK_WIDGET (self)->style, "ActiveTextColor", &style_active_color)) {
+		g_object_set (G_OBJECT (priv->account_name_renderer),
+			      "foreground-gdk", &style_active_color,
+			      NULL);
+		gtk_tree_view_column_set_attributes (priv->account_name_column, priv->account_name_renderer,
+						     "text", MODEST_ACCOUNT_VIEW_DISPLAY_NAME_COLUMN,
+						     "foreground-set", MODEST_ACCOUNT_VIEW_HAS_NEW_MAILS_COLUMN,
+						     NULL);
+	} else {
+		g_object_set (G_OBJECT (priv->account_name_renderer),
+			      "foreground-set", FALSE,
+			      "foreground-gdk", NULL,
+			      NULL);
+		gtk_tree_view_column_set_attributes (priv->account_name_column, priv->account_name_renderer,
+						     "text", MODEST_ACCOUNT_VIEW_DISPLAY_NAME_COLUMN,
+						     NULL);
+	}
+
+	update_account_view (priv->account_mgr, self);
+}
+
