@@ -51,6 +51,7 @@
 #include <modest-tny-folder.h>
 #include <tny-simple-list.h>
 #include <gdk/gdkkeysyms.h>
+#include <modest-isearch-toolbar.h>
 
 typedef enum {
 	CONTENTS_STATE_NONE = 0,
@@ -101,6 +102,8 @@ struct _ModestHeaderWindowPrivate {
 
 	/* weak refs */
 	GtkTreeModel *model_weak_ref;
+
+	GtkWidget   *isearch_toolbar;
 };
 #define MODEST_HEADER_WINDOW_GET_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE((o), \
 									  MODEST_TYPE_HEADER_WINDOW, \
@@ -180,10 +183,23 @@ static void on_header_view_model_destroyed (gpointer user_data,
 static gboolean on_key_press(GtkWidget *widget,
 					GdkEventKey *event,
 					gpointer user_data);
+static void on_screen_changed(GdkScreen *screen,
+		              gpointer userdata);
 #endif
 
+static void  isearch_toolbar_close  (GtkWidget *widget,
+				     ModestHeaderWindow *obj);
+static void  isearch_toolbar_search (GtkWidget *widget,
+				     ModestHeaderWindow *obj);
+#ifndef MODEST_TOOLKIT_HILDON2
+static void  show_isearch_toolbar   (GtkWidget *obj, gpointer data);
+static void  toggle_isearch_toolbar (GtkWidget *obj,
+				     gpointer data);
+#endif
+
+
 /* globals */
-static GtkWindowClass *parent_class = NULL;
+static ModestWindowParentClass *parent_class = NULL;
 
 #define EMPTYVIEW_XALIGN 0.5
 #define EMPTYVIEW_YALIGN 0.5
@@ -278,7 +294,29 @@ modest_header_window_dispose (GObject *obj)
 
 	folder = modest_header_view_get_folder ((ModestHeaderView *) priv->header_view);
 	if (folder) {
-		tny_folder_sync_async (folder, TRUE, NULL, NULL, NULL);
+		gboolean expunge;
+		TnyAccount *account;
+		ModestProtocolType proto_type;
+		ModestProtocolRegistry *proto_reg;
+		
+		account = modest_tny_folder_get_account (folder);
+		proto_type = modest_tny_account_get_protocol_type (account);
+		g_object_unref (account);
+		proto_reg = modest_runtime_get_protocol_registry ();
+		
+		if (modest_protocol_registry_protocol_type_has_leave_on_server (proto_reg, proto_type)) {
+			const gchar *name;
+			ModestAccountMgr* account_mgr;
+			
+			account_mgr = modest_runtime_get_account_mgr ();
+			name = modest_tny_account_get_parent_modest_account_name_for_server_account (account);
+			expunge = !modest_account_mgr_get_leave_on_server (account_mgr, name);
+		}
+		else {
+			expunge = TRUE;
+		}
+
+		tny_folder_sync_async (folder, expunge, NULL, NULL, NULL);
 		g_object_unref (folder);
 	}
 
@@ -348,6 +386,7 @@ modest_header_window_disconnect_signals (ModestWindow *self)
 static void
 connect_signals (ModestHeaderWindow *self)
 {
+	GdkScreen *screen;
 	ModestHeaderWindowPrivate *priv = MODEST_HEADER_WINDOW_GET_PRIVATE(self);
 
 	/* header view */
@@ -419,6 +458,14 @@ connect_signals (ModestHeaderWindow *self)
 
 
 #ifdef MODEST_TOOLKIT_HILDON2
+	screen = gtk_widget_get_screen(GTK_WIDGET(self));
+
+	priv->sighandlers = 
+		modest_signal_mgr_connect (priv->sighandlers,
+					   G_OBJECT (screen), 
+					   "size-changed",
+			     		    G_CALLBACK (on_screen_changed), self);
+
 	g_signal_connect(G_OBJECT(self), "key-press-event",
 			G_CALLBACK(on_key_press), self);
 #endif
@@ -433,54 +480,6 @@ folder_refreshed_cb (ModestMailOperation *mail_op,
 	update_view (MODEST_HEADER_WINDOW (user_data), NULL);
 }
 
-#ifdef MAEMO_CHANGES
-static gboolean
-tap_and_hold_query_cb (GtkWidget *header_view,
-		       GdkEvent *event,
-		       gpointer user_data)
-{
-	ModestHeaderWindow *self;
-	ModestHeaderWindowPrivate *priv;
-
-	self = (ModestHeaderWindow *) user_data;
-	priv = MODEST_HEADER_WINDOW_GET_PRIVATE (self);
-
-	if (event->type == GDK_BUTTON_PRESS) {
-		TnyHeader *header;
-
-		priv->x_coord = ((GdkEventButton*)event)->x;
-		priv->y_coord = ((GdkEventButton*)event)->y;
-
-		/* Enable/Disable mark as (un)read */
-		header = modest_header_view_get_header_at_pos ((ModestHeaderView *) header_view,
-							       priv->x_coord, priv->y_coord);
-		if (header) {
-			GList *children;
-			GtkWidget *mark_read_item, *mark_unread_item;
-
-			/* Show "mark as read" or "mark as unread" */
-			children = gtk_container_get_children (GTK_CONTAINER (priv->csm_menu));
-			mark_read_item = (GtkWidget *) g_list_nth_data (children, 1);
-			mark_unread_item = (GtkWidget *) g_list_nth_data (children, 2);
-
-			if (tny_header_get_flags (header) & TNY_HEADER_FLAG_SEEN) {
-				gtk_widget_show (mark_unread_item);
-				gtk_widget_hide (mark_read_item);
-			} else {
-				gtk_widget_show (mark_read_item);
-				gtk_widget_hide (mark_unread_item);
-			}
-			g_object_unref (header);
-		} else {
-			/* Do not show the CSM if there is no header below */
-			return TRUE;
-		}
-	}
-
-	return FALSE;
-}
-#endif
-
 static void
 delete_header (GtkWindow *parent,
 	       TnyHeader *header)
@@ -530,7 +529,8 @@ on_delete_csm_activated (GtkMenuItem *item,
 	header = modest_header_view_get_header_at_pos ((ModestHeaderView *) priv->header_view,
 						       priv->x_coord, priv->y_coord);
 	if (header) {
-		delete_header ((GtkWindow *) self, header);
+		GtkWindow *toplevel = (GtkWindow *) gtk_widget_get_toplevel ((GtkWidget *) self);
+		delete_header ((GTK_WIDGET_TOPLEVEL (toplevel)) ? toplevel : NULL, header);
 		g_object_unref (header);
 	}
 }
@@ -550,7 +550,11 @@ on_mark_read_csm_activated (GtkMenuItem *item,
 						       priv->x_coord, priv->y_coord);
 
 	if (header) {
+		gchar *uid;
 		tny_header_set_flag (header, TNY_HEADER_FLAG_SEEN);
+		uid = modest_tny_folder_get_header_unique_id (header);
+		modest_platform_emit_msg_read_changed_signal (uid, TRUE);
+		g_free (uid);
 		g_object_unref (header);
 	}
 }
@@ -570,7 +574,11 @@ on_mark_unread_csm_activated (GtkMenuItem *item,
 						       priv->x_coord, priv->y_coord);
 
 	if (header) {
+		gchar *uid;
 		tny_header_unset_flag (header, TNY_HEADER_FLAG_SEEN);
+		uid = modest_tny_folder_get_header_unique_id (header);
+		modest_platform_emit_msg_read_changed_signal (uid, FALSE);
+		g_free (uid);
 		g_object_unref (header);
 	}
 }
@@ -615,6 +623,91 @@ on_header_view_model_changed (GObject *gobject,
 	g_object_weak_ref ((GObject *) model, on_header_view_model_destroyed, self);
 }
 
+/* Returns TRUE if the user clicked over a valid TnyHeader instance */
+static gboolean
+show_context_menu (ModestHeaderWindow *self,
+		   GdkEventButton     *event)
+{
+	ModestHeaderWindowPrivate *priv;
+	TnyHeader *header;
+
+	priv = MODEST_HEADER_WINDOW_GET_PRIVATE (self);
+
+	priv->x_coord = event->x;
+	priv->y_coord = event->y;
+
+	/* Enable/Disable mark as (un)read */
+	header = modest_header_view_get_header_at_pos ((ModestHeaderView *) priv->header_view,
+						       priv->x_coord, priv->y_coord);
+	if (header) {
+		GList *children;
+		GtkWidget *mark_read_item, *mark_unread_item;
+
+		/* Show "mark as read" or "mark as unread" */
+		children = gtk_container_get_children (GTK_CONTAINER (priv->csm_menu));
+		mark_read_item = (GtkWidget *) g_list_nth_data (children, 1);
+		mark_unread_item = (GtkWidget *) g_list_nth_data (children, 2);
+
+		if (tny_header_get_flags (header) & TNY_HEADER_FLAG_SEEN) {
+			gtk_widget_show (mark_unread_item);
+			gtk_widget_hide (mark_read_item);
+		} else {
+			gtk_widget_show (mark_read_item);
+			gtk_widget_hide (mark_unread_item);
+		}
+		g_object_unref (header);
+	} else {
+		return FALSE;
+	}
+	return TRUE;
+}
+
+
+#ifdef MAEMO_CHANGES
+static gboolean
+tap_and_hold_query_cb (GtkWidget *header_view,
+		       GdkEvent *event,
+		       gpointer user_data)
+{
+	/* Ignore other clicks but right button single clicks */
+	if (event->type != GDK_BUTTON_PRESS)
+		return FALSE;
+
+	return !show_context_menu ((ModestHeaderWindow *) user_data,
+				   (GdkEventButton *) event);
+}
+#else
+static gboolean
+on_button_press_cb (GtkWidget      *widget,
+		    GdkEventButton *event,
+		    gpointer        user_data)
+{
+	/* Ignore other clicks but right button single clicks */
+	if (event->type != GDK_BUTTON_PRESS || event->button != 3)
+		return FALSE;
+
+	if (show_context_menu ((ModestHeaderWindow *) user_data, event)) {
+		ModestHeaderWindowPrivate *priv;
+
+		priv = MODEST_HEADER_WINDOW_GET_PRIVATE (user_data);
+		gtk_menu_popup ((GtkMenu *) priv->csm_menu, NULL, NULL,
+				NULL, NULL, event->button, event->time);
+	}
+
+	return FALSE;
+}
+
+static void
+on_screen_changed (GdkScreen *screen,
+                   gpointer   userdata) 
+{
+	ModestHeaderWindow *self = (ModestHeaderWindow *) userdata;
+	ModestHeaderWindowPrivate *priv = MODEST_HEADER_WINDOW_GET_PRIVATE (self);
+
+	gtk_widget_queue_resize(priv->new_message_button);
+}
+#endif
+
 static GtkWidget *
 create_header_view (ModestWindow *self, TnyFolder *folder)
 {
@@ -653,6 +746,9 @@ create_header_view (ModestWindow *self, TnyFolder *folder)
 #ifdef MAEMO_CHANGES
 	g_signal_connect ((GObject *) header_view, "tap-and-hold-query",
 			  G_CALLBACK (tap_and_hold_query_cb), self);
+#else
+	g_signal_connect ((GObject *) header_view, "button-press-event",
+			  G_CALLBACK (on_button_press_cb), self);
 #endif
 	g_signal_connect ((GObject *) delete_item, "activate",
 			  G_CALLBACK (on_delete_csm_activated), self);
@@ -745,7 +841,10 @@ modest_header_window_new (TnyFolder *folder, const gchar *account_name, const gc
 	ModestServerAccountSettings *store_settings = NULL;
 	GtkWidget *alignment;
 	gchar *account_display_name = NULL;
-	
+#ifdef MODEST_TOOLKIT_HILDON2
+	GtkWidget *live_search;
+#endif
+
 	self  = MODEST_HEADER_WINDOW(g_object_new(MODEST_TYPE_HEADER_WINDOW, NULL));
 	priv = MODEST_HEADER_WINDOW_GET_PRIVATE(self);
 
@@ -765,6 +864,10 @@ modest_header_window_new (TnyFolder *folder, const gchar *account_name, const gc
 				  self);
 
 	priv->header_view  = create_header_view (MODEST_WINDOW (self), folder);
+#ifdef MODEST_TOOLKIT_HILDON2
+	live_search = modest_header_view_setup_live_search (MODEST_HEADER_VIEW (priv->header_view));
+	hildon_live_search_widget_hook (HILDON_LIVE_SEARCH (live_search), GTK_WIDGET (self), priv->header_view);
+#endif
 	priv->empty_view = create_empty_view (MODEST_WINDOW (self));
 
 	/* Transform the floating reference in a "hard" reference. We
@@ -799,6 +902,9 @@ modest_header_window_new (TnyFolder *folder, const gchar *account_name, const gc
 
         priv->top_vbox = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (alignment), priv->contents_view);
+#ifdef MODEST_TOOLKIT_HILDON2
+	gtk_box_pack_end (GTK_BOX (priv->top_vbox), live_search, FALSE, FALSE, 0);
+#endif
 	gtk_box_pack_end (GTK_BOX (priv->top_vbox), alignment, TRUE, TRUE, 0);
 
 	gtk_container_add (GTK_CONTAINER (self), priv->top_vbox);
@@ -848,6 +954,14 @@ modest_header_window_new (TnyFolder *folder, const gchar *account_name, const gc
 						  EDIT_MODE_CALLBACK (modest_ui_actions_on_edit_mode_move_to));
 #endif
 
+	priv->isearch_toolbar = modest_toolkit_factory_create_isearch_toolbar (modest_runtime_get_toolkit_factory (),
+									       NULL);
+	modest_window_add_toolbar (MODEST_WINDOW (self), GTK_TOOLBAR (priv->isearch_toolbar));
+	g_signal_connect (G_OBJECT (priv->isearch_toolbar), "isearch-close", 
+			  G_CALLBACK (isearch_toolbar_close), self);
+	g_signal_connect (G_OBJECT (priv->isearch_toolbar), "isearch-search", 
+			  G_CALLBACK (isearch_toolbar_search), self);
+
 
 	modest_window_set_active_account (MODEST_WINDOW (self), account_name);
 	modest_window_set_active_mailbox (MODEST_WINDOW (self), mailbox);
@@ -957,6 +1071,10 @@ static void setup_menu (ModestHeaderWindow *self)
 	modest_window_add_to_menu (MODEST_WINDOW (self), _("mcen_me_outbox_cancelsend"), NULL,
 				   MODEST_WINDOW_MENU_CALLBACK (modest_ui_actions_cancel_send),
 				   MODEST_DIMMING_CALLBACK (modest_ui_dimming_rules_on_cancel_sending_all));
+#ifndef MODEST_TOOLKIT_HILDON2
+	modest_window_add_to_menu (MODEST_WINDOW (self), _HL("wdgt_bd_search"), "<Control>f",
+				   MODEST_WINDOW_MENU_CALLBACK (toggle_isearch_toolbar), NULL);
+#endif
 }
 
 static void 
@@ -1475,4 +1593,70 @@ on_key_press(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
 
 	return FALSE;
 }
+
+static void
+on_screen_changed (GdkScreen *screen,
+                   gpointer   userdata) 
+{
+	ModestHeaderWindow *self = (ModestHeaderWindow *) userdata;
+	ModestHeaderWindowPrivate *priv = MODEST_HEADER_WINDOW_GET_PRIVATE (self);
+
+	gtk_widget_queue_resize(priv->new_message_button);
+}
 #endif
+
+#ifndef MODEST_TOOLKIT_HILDON2
+/* Used for the Ctrl+F accelerator */
+static void
+toggle_isearch_toolbar (GtkWidget *obj,
+			gpointer data)
+{
+	ModestHeaderWindow *window = MODEST_HEADER_WINDOW (data);
+	ModestHeaderWindowPrivate *priv = MODEST_HEADER_WINDOW_GET_PRIVATE (window);
+
+	if (GTK_WIDGET_VISIBLE (priv->isearch_toolbar)) {
+		isearch_toolbar_close (obj, data);
+       } else {
+		show_isearch_toolbar (obj, data);
+       }
+}
+
+/* Handler for menu option */
+static void
+show_isearch_toolbar (GtkWidget *obj,
+		      gpointer data)
+{
+	ModestHeaderWindow *window = MODEST_HEADER_WINDOW (data);
+	ModestHeaderWindowPrivate *priv = MODEST_HEADER_WINDOW_GET_PRIVATE (window);
+
+	gtk_widget_show (priv->isearch_toolbar);
+	modest_isearch_toolbar_highlight_entry (MODEST_ISEARCH_TOOLBAR (priv->isearch_toolbar), TRUE);
+}
+#endif
+
+/* Handler for click on the "X" close button in isearch toolbar */
+static void
+isearch_toolbar_close (GtkWidget *widget,
+		       ModestHeaderWindow *obj)
+{
+	ModestHeaderWindowPrivate *priv;
+
+	priv = MODEST_HEADER_WINDOW_GET_PRIVATE (obj);
+
+	/* Hide toolbar */
+	gtk_widget_hide (priv->isearch_toolbar);
+
+	modest_header_view_set_filter_string (MODEST_HEADER_VIEW (priv->header_view), NULL);
+}
+
+static void
+isearch_toolbar_search (GtkWidget *widget,
+			ModestHeaderWindow *obj)
+{
+	ModestHeaderWindowPrivate *priv = MODEST_HEADER_WINDOW_GET_PRIVATE (obj);
+
+	/* TODO: set filter */
+	modest_header_view_set_filter_string (MODEST_HEADER_VIEW (priv->header_view), 
+					      modest_isearch_toolbar_get_search (MODEST_ISEARCH_TOOLBAR (priv->isearch_toolbar)));
+
+}
